<?php

declare (strict_types=1);
namespace ScopedGtbabel\Dotenv\Parser;

use ScopedGtbabel\Dotenv\Exception\InvalidFileException;
use ScopedGtbabel\Dotenv\Util\Regex;
use ScopedGtbabel\GrahamCampbell\ResultType\Result;
use ScopedGtbabel\GrahamCampbell\ResultType\Success;
final class Parser implements \ScopedGtbabel\Dotenv\Parser\ParserInterface
{
    /**
     * Parse content into an entry array.
     *
     * @param string $content
     *
     * @throws \Dotenv\Exception\InvalidFileException
     *
     * @return \Dotenv\Parser\Entry[]
     */
    public function parse(string $content)
    {
        return \ScopedGtbabel\Dotenv\Util\Regex::split("/(\r\n|\n|\r)/", $content)->mapError(static function () {
            return 'Could not split into separate lines.';
        })->flatMap(static function (array $lines) {
            return self::process(\ScopedGtbabel\Dotenv\Parser\Lines::process($lines));
        })->mapError(static function (string $error) {
            throw new \ScopedGtbabel\Dotenv\Exception\InvalidFileException(\sprintf('Failed to parse dotenv file. %s', $error));
        })->success()->get();
    }
    /**
     * Convert the raw entries into proper entries.
     *
     * @param string[] $entries
     *
     * @return \GrahamCampbell\ResultType\Result<\Dotenv\Parser\Entry[],string>
     */
    private static function process(array $entries)
    {
        return \array_reduce($entries, static function (\ScopedGtbabel\GrahamCampbell\ResultType\Result $result, string $raw) {
            return $result->flatMap(static function (array $entries) use($raw) {
                return \ScopedGtbabel\Dotenv\Parser\EntryParser::parse($raw)->map(static function (\ScopedGtbabel\Dotenv\Parser\Entry $entry) use($entries) {
                    return \array_merge($entries, [$entry]);
                });
            });
        }, \ScopedGtbabel\GrahamCampbell\ResultType\Success::create([]));
    }
}
