<?php

declare (strict_types=1);
namespace ScopedGtbabel\Dotenv\Repository\Adapter;

use ScopedGtbabel\PhpOption\Option;
use ScopedGtbabel\PhpOption\Some;
final class ServerConstAdapter implements \ScopedGtbabel\Dotenv\Repository\Adapter\AdapterInterface
{
    /**
     * Create a new server const adapter instance.
     *
     * @return void
     */
    private function __construct()
    {
        //
    }
    /**
     * Create a new instance of the adapter, if it is available.
     *
     * @return \PhpOption\Option<\Dotenv\Repository\Adapter\AdapterInterface>
     */
    public static function create()
    {
        /** @var \PhpOption\Option<AdapterInterface> */
        return \ScopedGtbabel\PhpOption\Some::create(new self());
    }
    /**
     * Read an environment variable, if it exists.
     *
     * @param string $name
     *
     * @return \PhpOption\Option<string>
     */
    public function read(string $name)
    {
        /** @var \PhpOption\Option<string> */
        return \ScopedGtbabel\PhpOption\Option::fromArraysValue($_SERVER, $name)->filter(static function ($value) {
            return \is_string($value);
        });
    }
    /**
     * Write to an environment variable, if possible.
     *
     * @param string $name
     * @param string $value
     *
     * @return bool
     */
    public function write(string $name, string $value)
    {
        $_SERVER[$name] = $value;
        return \true;
    }
    /**
     * Delete an environment variable, if possible.
     *
     * @param string $name
     *
     * @return bool
     */
    public function delete(string $name)
    {
        unset($_SERVER[$name]);
        return \true;
    }
}
