<?php

declare (strict_types=1);
namespace ScopedGtbabel\Dotenv\Store;

use ScopedGtbabel\Dotenv\Exception\InvalidPathException;
use ScopedGtbabel\Dotenv\Store\File\Reader;
final class FileStore implements \ScopedGtbabel\Dotenv\Store\StoreInterface
{
    /**
     * The file paths.
     *
     * @var string[]
     */
    private $filePaths;
    /**
     * Should file loading short circuit?
     *
     * @var bool
     */
    private $shortCircuit;
    /**
     * The file encoding.
     *
     * @var string|null
     */
    private $fileEncoding;
    /**
     * Create a new file store instance.
     *
     * @param string[]    $filePaths
     * @param bool        $shortCircuit
     * @param string|null $fileEncoding
     *
     * @return void
     */
    public function __construct(array $filePaths, bool $shortCircuit, string $fileEncoding = null)
    {
        $this->filePaths = $filePaths;
        $this->shortCircuit = $shortCircuit;
        $this->fileEncoding = $fileEncoding;
    }
    /**
     * Read the content of the environment file(s).
     *
     * @throws \Dotenv\Exception\InvalidEncodingException|\Dotenv\Exception\InvalidPathException
     *
     * @return string
     */
    public function read()
    {
        if ($this->filePaths === []) {
            throw new \ScopedGtbabel\Dotenv\Exception\InvalidPathException('At least one environment file path must be provided.');
        }
        $contents = \ScopedGtbabel\Dotenv\Store\File\Reader::read($this->filePaths, $this->shortCircuit, $this->fileEncoding);
        if (\count($contents) > 0) {
            return \implode("\n", $contents);
        }
        throw new \ScopedGtbabel\Dotenv\Exception\InvalidPathException(\sprintf('Unable to read any of the environment file(s) at [%s].', \implode(', ', $this->filePaths)));
    }
}
